import { Injectable, Output, EventEmitter } from '@angular/core';
import { Headers, Http, Response, RequestOptions, URLSearchParams, ResponseContentType } from '@angular/http';
import 'rxjs/add/operator/toPromise';
import 'rxjs/Rx';
import { Observable } from 'rxjs/Observable';
import { Subject } from 'rxjs/Subject';
import { CMF } from '../shared-types/cmf';
import { FILTERSETTINGS } from '../shared-types/filter-settings';
import { BusyIndicatorService } from '../shared-services/busy-indicator.service';
import { DialogService } from '../shared-services/dialog.service';
import { SettingsService } from '../shared-services/settings.service';
import { CatchAllListsService } from '../shared-services/catch-all-lists.service';
import { CustomQueryEncoderHelper } from '../shared-helpers/customQueryEncoderHelper';
import { shim } from 'promise.prototype.finally';
import * as FileSaver from 'file-saver';
import { environment } from '../../environments/environment';

@Injectable()
export class SearchService {
    private searchCompleteSubject = new Subject<any>();
    resultsList: CMF[] = [];
    noMatches = false;
    hideSearchFilters = false;

    constructor(private http: Http, private busyIndicatorService: BusyIndicatorService, private dialogService: DialogService,
        private catchAllListsService: CatchAllListsService, private settingsService: SettingsService) {
        shim();
    }

    doSearch(filterSettings: FILTERSETTINGS, isBasic: boolean) {
        const postBody = this.getPostBody(filterSettings, isBasic);

        if (postBody.length === 0) {
            this.dialogService.showMissingSearchFilterDialog();
        } else {
            this.busyIndicatorService.showBusyIndicator('Searching, please wait...');

            const headers = new Headers({
                'Accept': 'application/json',
                'Content-Type': 'application/json'
            });

            // Set the request options.
            const requestOptions = new RequestOptions();
            requestOptions.headers = headers;
            requestOptions.withCredentials = true;

            let promise: Promise<CMF[]>;

            promise = this.http.post(environment.cmfSearchApiUrl, postBody, requestOptions)
                .toPromise()
                .then(response => response.json() as CMF[])
                .catch((err) => {
                    this.handleError(err, this.dialogService);
                })
                .finally(() => {
                    this.busyIndicatorService.hideBusyIndicator();
                });

            promise.then(results => {
                if (results) {
                    this.resultsList = results;
                    this.cleanupResults();
                    this.noMatches = this.resultsList.length === 0;
                    this.hideSearchFilters = this.resultsList.length !== 0;
                    this.searchCompleteSubject.next(true);
                }
            });
        }
    }

    doExport(filterSettings: FILTERSETTINGS, isBasic: boolean) {
        this.busyIndicatorService.showBusyIndicator('Downloading Excel file, please wait...');

        const postBody = this.getPostBody(filterSettings, isBasic);

        const headers = new Headers({
            'Accept': 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
            'Content-Type': 'application/json'
        });

        const requestOptions = new RequestOptions({ responseType: ResponseContentType.Blob });
        requestOptions.headers = headers;
        requestOptions.withCredentials = true;

        this.http.post(environment.excelExportApiUrl, postBody, requestOptions)
            .map(res => res.blob())
            .subscribe(
            data => {
                this.busyIndicatorService.hideBusyIndicator();
                FileSaver.saveAs(data, 'Owl-Export.xlsx');
            },
            err => {
                this.busyIndicatorService.hideBusyIndicator();
                console.log('Error Encountered In SearchService (doExport)');
                console.error(err);
            });
    }

    reset() {
        this.resultsList = [];
        this.noMatches = false;
        this.hideSearchFilters = false;
    }

    getSearchCompleteSubscription(): Observable<any> {
        return this.searchCompleteSubject.asObservable();
    }

    private cleanupResults() {
        // Put new-lines back into the Special Instructions fields.
        this.resultsList.forEach(x => {
            if (x.SPECIAL_INST) {
                x.SPECIAL_INST = x.SPECIAL_INST.replace(/\\n/g, '\n')
            }
        });

        // Set CreatedOn date.
        this.resultsList.forEach(x => {
            if (x.CUST_CRE_DT) {
                x.CreatedOn = new Date(Number(x.CUST_CRE_DT.replace(/\D/g, '')))
            }
        });
    }

    private handleError(error: any, dialogService: DialogService): Promise<any> {
        console.log('Error Encountered In SearchService (doSearch)');
        console.error(error);

        if (error.status === 500 && String(error._body).startsWith('Matches: ')) {
            const numMatches = Number(String(error._body).replace('Matches: ', ''));
            dialogService.showTooManyMatchesDialog(numMatches);
        } else {
            dialogService.showCommunicationErrorDialog();
        }
        return Promise.reject(error.message || error);
    }


    private getPostBody(filterSettings: FILTERSETTINGS, isBasic: boolean): string {
        const searchCriteriaDict = {};
        let idx: number;

        if (isBasic === true) {
            if (filterSettings.CmfIdValue && filterSettings.CmfIdValue.trim()) {
                if (filterSettings.IsExactMatch === true) {
                    searchCriteriaDict['CmfIdExactMatch'] = filterSettings.CmfIdValue.trim();
                } else {
                    searchCriteriaDict['CmfId'] = filterSettings.CmfIdValue.trim();
                }
            }
        } else {
            if (filterSettings.CmfIdValue && filterSettings.CmfIdValue.trim()) {
                searchCriteriaDict['CmfId'] = filterSettings.CmfIdValue.trim();
            }

            if (filterSettings.PartnerFunction && filterSettings.PartnerFunction.trim() && filterSettings.PartnerFunction !== 'Any') {
                searchCriteriaDict['PartnerFunction'] = filterSettings.PartnerFunction.trim();
            }

            if (filterSettings.StreetAddress && filterSettings.StreetAddress.trim()) {
                searchCriteriaDict['Address'] = filterSettings.StreetAddress.trim();
            }

            if (filterSettings.CustomerName && filterSettings.CustomerName.trim()) {
                searchCriteriaDict['CustomerName'] = filterSettings.CustomerName.trim();
            }

            if (filterSettings.City && filterSettings.City.trim()) {
                searchCriteriaDict['City'] = filterSettings.City.trim();
            }

            if (filterSettings.State && filterSettings.State.trim() && filterSettings.State !== 'Any') {
                searchCriteriaDict['StateProvince'] = filterSettings.State.trim();
            }

            if (filterSettings.Country && filterSettings.Country.trim() && filterSettings.Country !== 'Any') {
                // Country will be set to something like. "United Status (US)", and we just want to pass
                // the two letter country code, which is "US".
                idx = filterSettings.Country.trim().indexOf('(');
                searchCriteriaDict['CountryCode'] = filterSettings.Country.trim().slice(idx + 1, idx + 3);
            }

            if (filterSettings.BusinessPurpose && filterSettings.BusinessPurpose.trim() && filterSettings.BusinessPurpose !== 'Any') {
                searchCriteriaDict['BusinessPurpose'] = filterSettings.BusinessPurpose.trim();
            }

            if (filterSettings.SalesOrgs && filterSettings.SalesOrgs.trim() && filterSettings.SalesOrgs !== 'Any') {
                searchCriteriaDict['SalesOrgsStr'] = filterSettings.SalesOrgs.trim();
            }

            if (filterSettings.DistributionChannel && filterSettings.DistributionChannel.trim() &&
                filterSettings.DistributionChannel !== 'Any') {
                searchCriteriaDict['DistributionChannel'] = filterSettings.DistributionChannel.trim();
            }

            if (filterSettings.SalesOffice && filterSettings.SalesOffice.trim() && filterSettings.SalesOffice !== 'Any') {
                searchCriteriaDict['SalesOffice'] = filterSettings.SalesOffice.trim();
            }

            if (filterSettings.SalesGroup && filterSettings.SalesGroup.trim() && filterSettings.SalesGroup !== 'Any') {
                searchCriteriaDict['SalesGroup'] = filterSettings.SalesGroup.trim();
            }

            if (filterSettings.SearchTerm && filterSettings.SearchTerm.trim()) {
                searchCriteriaDict['SearchTerm'] = filterSettings.SearchTerm.trim();
            }

            if (filterSettings.IndustryKeyCode && filterSettings.IndustryKeyCode.trim() && filterSettings.IndustryKeyCode !== 'Any') {
                searchCriteriaDict['Industry'] = filterSettings.IndustryKeyCode.trim();
            }

            if (filterSettings.CustomerGroupCode1 && filterSettings.CustomerGroupCode1.trim() &&
                filterSettings.CustomerGroupCode1 !== 'Any') {
                searchCriteriaDict['CustomerGroupCode1'] = filterSettings.CustomerGroupCode1.trim();
            }

            if (filterSettings.CustomerGroupCode2 && filterSettings.CustomerGroupCode2.trim() &&
                filterSettings.CustomerGroupCode2 !== 'Any') {
                searchCriteriaDict['CustomerGroupCode2'] = filterSettings.CustomerGroupCode2.trim();
            }

            if (filterSettings.CustomerGroupCode3 && filterSettings.CustomerGroupCode3.trim() &&
                filterSettings.CustomerGroupCode3 !== 'Any') {
                searchCriteriaDict['CustomerGroupCode3'] = filterSettings.CustomerGroupCode3.trim();
            }

            if (filterSettings.CustomerGroupCode4 && filterSettings.CustomerGroupCode4.trim() &&
                filterSettings.CustomerGroupCode4 !== 'Any') {
                searchCriteriaDict['CustomerGroupCode4'] = filterSettings.CustomerGroupCode4.trim();
            }

            if (filterSettings.CustomerGroupCode5 && filterSettings.CustomerGroupCode5.trim() &&
                filterSettings.CustomerGroupCode5 !== 'Any') {
                searchCriteriaDict['CustomerGroupCode5'] = filterSettings.CustomerGroupCode5.trim();
            }

            if (filterSettings.Incoterm1 && filterSettings.Incoterm1.trim() && filterSettings.Incoterm1 !== 'Any') {
                searchCriteriaDict['Incoterms1'] = filterSettings.Incoterm1.trim();
            }

            if (filterSettings.Incoterm2 && filterSettings.Incoterm2.trim()) {
                searchCriteriaDict['Incoterms2'] = filterSettings.Incoterm2.trim();
            }

            if (filterSettings.PlantCode && filterSettings.PlantCode.trim() && filterSettings.PlantCode !== 'Any') {
                searchCriteriaDict['PlantCode'] = filterSettings.PlantCode.trim();
            }

            if (filterSettings.Attribute8 && filterSettings.Attribute8.trim() && filterSettings.Attribute8 !== 'Any') {
                searchCriteriaDict['Attribute8'] = filterSettings.Attribute8.trim();
            }
        }

        // If the user did not specify any search critieria, bail.
        if (Object.keys(searchCriteriaDict).length === 0) {
            return '';
        }

        const selectedEnvironmentsDict = {
            'DatabaseEnv': this.settingsService.selectedDatabaseEnv.Id.toString(),
            'SharepointEnv': this.settingsService.selectedSharepointEnv.Id.toString()
        };

        const myJaggedArray = {
            'SelectedEnvironments': selectedEnvironmentsDict,
            'SearchCriteria': searchCriteriaDict
        };

        return JSON.stringify(myJaggedArray);
    }
}
